@@ -4,7 +4,7 @@ from http import HTTPStatus
 from itertools import chain
 from json import loads, JSONDecodeError
 from logging import getLogger
-from typing import Dict
+from typing import Dict, List
 
 from django.conf import settings
 from django.contrib.admin.views.decorators import staff_member_required
@@ -13,9 +13,11 @@ from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseNotFou
 from django.shortcuts import get_object_or_404, redirect, render
 from django.template.defaultfilters import linebreaksbr
 from django.utils import timezone, formats
+from django.utils.safestring import mark_safe
 from django.views.decorators.http import require_GET, require_POST
 
 from NEMO import rates
+from NEMO.exceptions import RequiredUnansweredQuestionsException
 from NEMO.forms import CommentForm, nice_errors
 from NEMO.models import (
 	Comment,
@@ -145,15 +147,27 @@ def usage_data_history(request, tool_id):
 	""" This method return a dictionary of headers and rows containing run_data information for Usage Events """
 	csv_export = bool(request.POST.get("csv", False))
 	start, end = extract_times(request.POST, start_required=False, end_required=False)
-	if not start and not end:
-		# Default to last 30 days of usage data
-		end = timezone.now().astimezone(timezone.get_current_timezone())
-		start = end + timedelta(days=-30)
+	last = request.POST.get("data_history_last")
+	user_id = request.POST.get("data_history_user_id")
+	if not last and not start and not end:
+		# Default to last 25 records
+		last = 25
 	usage_events = UsageEvent.objects.filter(tool_id=tool_id, end__isnull=False).order_by("-end")
 	if start:
 		usage_events = usage_events.filter(end__gte=beginning_of_the_day(start))
 	if end:
 		usage_events = usage_events.filter(end__lte=end_of_the_day(end))
+	if user_id:
+		try:
+			usage_events = usage_events.filter(user_id=int(user_id))
+		except ValueError:
+			pass
+	if last:
+		try:
+			last = int(last)
+		except ValueError:
+			last = 25
+		usage_events = usage_events[:last]
 	table_result = BasicDisplayTable()
 	table_result.add_header(("user", "User"))
 	table_result.add_header(("date", "Date"))
@@ -194,7 +208,7 @@ def usage_data_history(request, tool_id):
 				tool_control_logger.debug("error decoding run_data: " + usage_event.run_data)
 	if csv_export:
 		response = table_result.to_csv()
-		filename = f"usage_data_{start.strftime('%m_%d_%Y')}_to_{end.strftime('%m_%d_%Y')}.csv"
+		filename = f"tool_usage_data_export_{datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}.csv"
 		response["Content-Disposition"] = f'attachment; filename="{filename}"'
 		return response
 	else:
@@ -202,7 +216,10 @@ def usage_data_history(request, tool_id):
 			"tool_id": tool_id,
 			"data_history_start": start,
 			"data_history_end": end,
+			"data_history_last": str(last),
 			"usage_data_table": table_result,
+			"data_history_user": User.objects.get(id=user_id) if user_id else None,
+			"users": User.objects.filter(is_active=True)
 		}
 		return render(request, "tool_control/usage_data.html", dictionary)
 
@@ -368,8 +385,13 @@ def disable_tool(request, tool_id):
 
 	try:
 		current_usage_event.run_data = dynamic_form.extract(request)
-	except Exception as e:
-		return HttpResponseBadRequest(str(e))
+	except RequiredUnansweredQuestionsException as e:
+		if request.user.is_staff and request.user != current_usage_event.operator and current_usage_event.user != request.user:
+			# if a staff is forcing somebody off the tool and there are required questions, send an email and proceed
+			current_usage_event.run_data = e.run_data
+			email_managers_required_questions_disable_tool(current_usage_event.operator, request.user, tool, e.questions)
+		else:
+			return HttpResponseBadRequest(str(e))
 
 	dynamic_form.charge_for_consumables(
 		current_usage_event.user,
@@ -495,3 +517,23 @@ def interlock_error(action:str, user:User):
 		"action": action
 	}
 	return JsonResponse(dictionary, status=501)
+
+
+def email_managers_required_questions_disable_tool(tool_user:User, staff_member:User, tool:Tool, questions:List[PostUsageQuestion]):
+	abuse_email_address = get_customization('abuse_email_address')
+	managers = []
+	if hasattr(settings, 'LAB_MANAGERS'):
+		managers = settings.LAB_MANAGERS
+	ccs = set(tuple([r for r in [staff_member.email, tool.primary_owner.email, *tool.backup_owners.all().values_list('email', flat=True), *managers] if r]))
+	display_questions = "".join([linebreaksbr(mark_safe(question.render_as_text())) + "<br/><br/>" for question in questions])
+	message = f"""
+Dear {tool_user.get_name()},<br/>
+You have been logged off by staff from the {tool} that requires answers to the following post-usage questions:<br/>
+<br/>
+{display_questions}
+<br/>
+Regards,<br/>
+<br/>
+NanoFab Management<br/>
+"""
+	send_mail(subject=f"Unanswered post‑usage questions after logoff from the {tool.name}", content=message, from_email=abuse_email_address, to=[tool_user.email], cc=ccs, email_category=EmailCategory.ABUSE)
